# Filename:     tb_tflipflop.do
# Filetype:     Modelsim Command File
# Date:         28 feb 2014
# Update:       
# Description:  Command file for simulating the T flipflop
# Author:       J. op den Brouw
# State:        demo
# Error:        -
# Version:      1.0
# Copyright:    (c)2014, De Haagse Hogeschool

# Set transcript on
transcript on

# Recreate the work directory and map to work
if {[file exists rtl_work]} {
    vdel -lib rtl_work -all
}
vlib rtl_work
vmap work rtl_work

# Find out if we're started through Quartus or by hand
# (or by using an exec in the Tcl window in Quartus).
# Quartus has the annoying property that it will start
# Modelsim from a directory called "simulation/modelsim".
# The design and the testbench are located in the project
# root, so we've to compensate for that.
if [ string match "*simulation/modelsim" [pwd] ] { 
	set prefix "../../"
	puts "Running Modelsim from Quartus..."
} else {
	set prefix ""
	puts "Running Modelsim..."
}

# Compile the VHDL description and testbench,
# please note that the design and its testbench are located
# in the project root, but the simulator start in directory
# <project_root>/simulation/modelsim, so we have to compensate
# for that.
vcom -93 -work work ${prefix}tflipflop.vhd
vcom -93 -work work ${prefix}tb_tflipflop.vhd

# Start the simulator with 1 ns time resolution.
# Change the last item name to "sim_alt" (without the double quotes)
# to use the alternative testbench.
vsim -t 1ns -L rtl_work -L work -voptargs="+acc" tb_tflipflop sim_alt

# Log all signals in the design, good if the number
# of signals is small.
add log -r *

# Add all toplevel signals to the list view
# Add a number of signals of the simulated design
add list clk
add list areset
add list t
add list q

# Add all toplevel signals to the wave view
# Add a number of signals inside the simulated design
add wave -divider "Inputs"
add wave clk
add wave areset
add wave t
add wave -divider "Outputs"
add wave q

# Open Structure, List and Waveform window
view structure
view list
view wave

# Run simulation until no more events on stimuli
run 470 ns

# Fill up the waveform in the window
wave zoom full
