-- Filename:     tb_tflipflop.vhd
-- Filetype:     VHDL Test Bench File
-- Date:         28 feb 2014
-- Update:       -
-- Description:  Test bench for tflipflop
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.0
-- Copyright:    (c)2014, De Haagse Hogeschool
--
-- This is a test bench for the T fliflop
-- VHDL description.

library ieee;
use ieee.std_logic_1164.all;

-- Empty entity
entity tb_tflipflop is
end entity tb_tflipflop;

-- The architecture (sim)
architecture sim of tb_tflipflop is
-- These are the signals we want to monitor
signal clk    : std_logic;
signal areset : std_logic;
signal t      : std_logic;
signal q      : std_logic;

-- The T flipflop...
component tflipflop is
port (clk    : in std_logic;
      areset : in std_logic;
      t      : in std_logic;
      q      : out std_logic
     );
end component tflipflop;
constant Tperiod : time := 20 ns;
constant small_delay : time := Tperiod / 10;
begin

    -- Instantiate a T flipflop and map the port signals
    -- to the monitoring signals
    dut : entity work.tflipflop(behavioral_alt)
    port map (clk => clk, areset => areset, t => t, q => q);

    clockgen: process is  -- no sensitivity list
    begin
        clk <= '0';         -- clock has 50% Duty Cyle
        wait for Tperiod/2;
        clk <= '1';
        wait for Tperiod/2;
    end process clockgen;

   -- The stimuli. We loop through all combinations
    datagen: process is
    begin
        t <= '0';
        areset <= '1';
        wait for 2*Tperiod;     -- wait for two clock cycles
 
        areset <= '0';
        wait for 2*Tperiod;
        wait until clk = '1';   -- synchronize on positive edge
        wait for small_delay;   -- and wait a bit

        t <= '1';               -- set value
        wait until clk = '1';   -- wait for four clocks
        wait until clk = '1';
        wait until clk = '1';
        -- wait until clk = '1';
        wait for small_delay;

        t <= '0';
        -- wait until clk = '1';
        -- wait until clk = '1';
        -- wait until clk = '1';
        -- wait until clk = '1';
        -- wait for small_delay;

        -- wait forever
        wait;
    end process;
 
end architecture sim;

architecture sim_alt of tb_tflipflop is

-- Simulation time constants
constant Tperiod : time := 20 ns;
constant small_delay : time := Tperiod / 10;

-- These are the signals we want to monitor
signal clk    : std_logic;
signal areset : std_logic;
signal t      : std_logic;
signal q      : std_logic;

-- The T flipflop...
component tflipflop is
port (clk    : in std_logic;
      areset : in std_logic;
      t      : in std_logic;
      q      : out std_logic
     );
end component tflipflop;
begin

    -- Instantiate a T flipflop and map the port signals
    -- to the monitoring signals
    dut : tflipflop
    port map (clk => clk, areset => areset, t => t, q => q);

    clockgen: process is  -- no sensitivity list
    begin
        clk <= '0';         -- clock has 50% Duty Cyle
        wait for Tperiod/2;
        clk <= '1';
        wait for Tperiod/2;
    end process clockgen;

   datagen: process is
   -- the valuations for the T-flipflop as a string
   constant t_i_s : std_logic_vector := "1111110101101001110";
   -- range is from 0 to 18 (19 elements)
   variable t_i : std_logic_vector(t_i_s'range) := t_i_s;
   begin

        t <= '0';
        areset <= '1';
        wait for 2*Tperiod;     -- wait for two clock cycles
 
        areset <= '0';
        wait for 2*Tperiod;
        wait until clk = '1';   -- synchronize on positive edge
        wait for small_delay;   -- and wait a bit

        for index in t_i'range loop   -- loop through all input bits
            -- report "The value of 'index' is " & integer'image(index);
            t <= t_i(index);          -- assign to T-flipflop input
            wait until clk = '1';
            wait for small_delay;
        end loop;

        -- wait forever
        wait;
    end process;
 
end architecture sim_alt;
