-- Filename:     tflipflop.vhd
-- Filetype:     VHDL Design File
-- Date:         28 feb 2014
-- Update:       -
-- Description:  VHDL description of a T-flipflop
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.0
-- Copyright:    (c)2014, De Haagse Hogeschool
--
-- This is a VHDL description of a T-fliflop.

library ieee;
use ieee.std_logic_1164.all;

-- The T-flipflop (Toggle Flipflop)
entity tflipflop is
port (clk    : in std_logic;
      areset : in std_logic;
      t      : in std_logic;
      q      : out std_logic
     );
end entity tflipflop;

-- The architecture
architecture behavioral of tflipflop is
signal q_int : std_logic;
begin

    process (clk, areset) is
    begin
        if areset = '1' then
            q_int <= '0';
        elsif rising_edge(clk) then
            q_int <= q_int xor t;
        end if;
    end process;

    q <= q_int;

end architecture behavioral;

-- An alternative architecture
architecture behavioral_alt of tflipflop is
signal q_int : std_logic;
begin

    process (clk, areset, q_int) is
    begin
        if areset = '1' then
            q_int <= '0';
        elsif rising_edge(clk) then
            if t = '1' then          -- If T = 1 ...
                q_int <= not q_int;  -- ... then toggle ...
            end if;
        end if;
        q <= q_int;

    end process;

end architecture behavioral_alt;